import json
import glob
from docx import Document


def create_report(data, title, path):
    document = Document()
    document.add_heading(title, level=0)

    for table in data:
        tab_data = data[table]
        document.add_heading(table)
        document.add_heading('Records', level=2)

        document.add_paragraph('Table contains {} records. {} of those records are fully populated. '
                               '{} records are partially filled. '
                               '{} records have no data at all.'.format(tab_data['No. records'],
                                                                        tab_data['No. full records'],
                                                                        tab_data['No. partial records'],
                                                                        tab_data['No. empty records']))

        document.add_heading('Attributes', level=2)
        document.add_paragraph('The following attributes are not populated, or are inaccessible, in all records.')
        for a, v in tab_data['attribute populations'].items():
            if v == 0:
                document.add_paragraph(a, style='List Bullet')
        document.add_paragraph('The following attributes are populated in the following quantities.')
        atts = [a for a, v in tab_data['attribute populations'].items() if v > 0]
        quan = [v for a, v in tab_data['attribute populations'].items() if v > 0]
        atts = [a for _, a in reversed(sorted(zip(quan, atts)))]
        quan = [q for q in reversed(sorted(quan))]
        rows = len(quan) + 1
        doctab = document.add_table(rows=rows, cols=2)
        doctab.style = 'Light Shading Accent 1'
        doctab.rows[0].cells[0].text = 'Attributes'
        doctab.rows[0].cells[1].text = 'Number of records'
        for a, q, r in zip(atts, quan, doctab.rows[1:]):
            cells = r.cells
            cells[0].text = a
            cells[1].text = str(q)
        document.add_heading('Links', level=2)
        document.add_paragraph('The following link groups are not populated in any record.')
        for a, v in tab_data['link populations'].items():
            if v == 0:
                document.add_paragraph(a, style='List Bullet')
        document.add_paragraph('The following table shows how many links there are associated with each link group '
                               'in this table.')
        lnks = [a for a, v in tab_data['link populations'].items() if v > 0]
        quan = [v for a, v in tab_data['link populations'].items() if v > 0]
        lnks = [a for _, a in reversed(sorted(zip(quan, lnks)))]
        quan = [q for q in reversed(sorted(quan))]
        rows = len(quan) + 1
        doctab2 = document.add_table(rows=rows, cols=2)
        doctab2.style = 'Light Shading Accent 1'
        doctab2.rows[0].cells[0].text = 'Link Group Name'
        doctab2.rows[0].cells[1].text = 'Number of links'
        for k, q, r in zip(lnks, quan, doctab2.rows[1:]):
            cells = r.cells
            cells[0].text = k
            cells[1].text = str(q)
    document.save(path+'{}.docx'.format(title))


if __name__ == '__main__':
    all_results_files = glob.glob('./*.json')
    for file_name in all_results_files:
        with open(file_name, 'r') as f:
            results = json.load(f)
            create_report(results, file_name[2:-5], path='./')
